/**
 * @author Torgie
 */

document.write('<link rel="stylesheet" href="engine/editor.css" />');

(function(window, engine) {

// Only enter edit mode if requested by the user
if(location.search.match(/editMode/) != null) {

function editor() {

	// Setting a "self" object for scoping issues.
	// See brothercake's reply here: http://www.codingforums.com/archive/index.php/t-49047.html
	var self = this;

	// Which layers will draw on the render pass
	this.drawLayers = {
		background5:	true,
		background4:	true,
		background3:	true,
		background2:	true,
		background1:	true,
		game:			true,
		foreground:		true,
	};

	// Let the engine settle down before turning on debugging
	this.initInterval = setInterval(function() {
		// When the first loop is called we can safely hook in the editor listeners
		if(engine.logicCalls == 0)
			return;

		clearInterval(self.initInterval);

		// Bind up the mouse events for the game screen
		self.bindInputEvents();

		engine.debug("Editor loaded and events have been bound.");
	}, 100);

	/**
	 * Seeks out the window object and gathers a list of every known sprite we've declared so far
	 */
	this.knownSprites = [];
	this.knownAudio = [];
	this.gatherKnownItems = function() {
		// Gather all sprites from the window object
		var temp = [];
		for(var i in window)
			if(i.match(/sprite_/) != null)
				temp.push(i);
		this.knownSprites = temp.sort();

		// Now do audio
		temp = [];
		for(var i in engine.audio)
			temp.push(i);
		this.knownAudio = temp.sort();
	};//end function gatherKnownItems

	/**
	 * Binds the mouse events INTO the gameState object. Pay close attention to the last 3 lines of this function, as they show how
	 * the functions are assigned straight into the gameState. That means that even though the function appear to have "this." and
	 * appear to be in the editor context, they are not.
	 */
	this.engineMousedown = this.engineMousemove = this.engineMouseup = false;
	this.bindInputEvents = function() {
		// Override the engine's mouse mapping
		engine.gs.showDebugging = true;
		engine.gs.mouseControl = true;
		engine.gs.keyControl = true;

		// Back up the original gamestate event hooks
		this.engineMousedown = engine.gs.mousedownEvent;
		this.engineMousemove = engine.gs.mousemoveEvent;
		this.engineMouseup = engine.gs.mouseupEvent;

		// Override the handlers to do fancy things
		engine.gs.mousedownEvent = this.mousedownEvent;
		engine.gs.mousemoveEvent = this.mousemoveEvent;
		engine.gs.mouseupEvent = this.mouseupEvent;
	};//end function bindMouseEvents

	this.clickedOnSelection = false;
	// Reminder: in the gameState context
	this.mousedownEvent = function(event) {
		this.mouseDown = true;

		// Figure out if the mouseDown happened in a selection
		self.clickedOnSelection = false;
		if(self.currentSelection.length > 0 && !engine.gs.keySpace) {
			for(var i in self.currentSelection) {
				var s = self.currentSelection[i];

				// Get the x, y, width, height of our test sprite (borrowed from engine.seekSprite())
				var renderPoint = s.getRenderPoint(engine.gs);
				var width = s.renderWidth;
				var height = s.renderHeight;

				// Compare and see if the click lands inside the bounding box
				if (event.offsetX >= renderPoint.x && event.offsetX <=  renderPoint.x + width && event.offsetY >= renderPoint.y && event.offsetY <= renderPoint.y + height)
					self.clickedOnSelection = true;
			}
		}

		this.mouseDownX = this.mouseLastX = event.offsetX;
		this.mouseDownY = this.mouseLastY = event.offsetY;
	};//end function mousedownEvent

	// Reminder: in the gameState context
	this.mousemoveEvent = function(event) {

		this.mouseX = event.offsetX;
		this.mouseY = event.offsetY;

		// If we're clicked AND the mouse has moved, start a drag event
		if (this.mouseDown &&
			(this.mouseLastX != this.mouseX || this.mouseLastY != this.mouseY)) {

			// Indicate the start of a drag event, and calculate how far it will be
			this.mouseDrag = true;
			var deltaX = this.mouseLastX - this.mouseX;
			var deltaY = this.mouseLastY - this.mouseY;

			// If the mouse is over a selection, move the selection instead of the camera
			if (self.clickedOnSelection) {
				self.moveSelection(deltaX * -1, deltaY * -1);
			}
			else {
				engine.gs.cameraX += deltaX;
				engine.gs.cameraY += deltaY;
			}

			$("#updateCamera_x").val(engine.gs.cameraX);
			$("#updateCamera_y").val(engine.gs.cameraY);

			// Update our last mouse position
			this.mouseLastX = this.mouseX;
			this.mouseLastY = this.mouseY;
		}
	};//end function mousemoveEvent

	this.currentSelection = [];
	// Reminder: in the gameState context
	this.mouseupEvent = function(event) {
		if(this.mouseDown && !this.mouseDrag) {
			// Gather all the sprites under the mouse. Note the use of "self" here to get into the editor context
			selection = engine.seekSprite(event.offsetX, event.offsetY);

			// If we're holding control, add the selected items to the current selection
			if(event.ctrlKey) {
				// Avoid duplicates - go through each item in self.currentSelection and test for matching IDs
				for(var i in selection) {
					var match = false;
					for(var j in self.currentSelection)
						if(selection[i].id == self.currentSelection[j].id)
							match = true;
					// If no match was found, add the item to the currentSelection
					if(!match)
						self.currentSelection.push(selection[i]);
				}
			}
			else
				self.currentSelection = selection;

			// Update the context menu with our new selection
			self.updateContextTools();
		}

		// Clear out the mouse tracking information
		this.mouseDown = this.mouseDrag = false;
		this.mouseLastX = this.mouseLastY = 0;
	};//end function mouseupEvent

	this.updateContextTools = function() {
		// Start building up the selection tool
		$("#section_selectionInfo").html("");

		// If no sprites are present, empty out the tool
		if (self.currentSelection.length == 0) {
			$("#section_selectionInfo").append('<i style="color: #AAA;">No sprites selected</i>');
			return;
		}

		// Go through each sprite and add it to the selection tool as well as highlight it
		for (var i in self.currentSelection) {
			var s = self.currentSelection[i];
			s.selected = true;

			var w = s.renderWidth;
			var h = s.renderHeight;

			// Add in the HTML for the context menu
			$("#section_selectionInfo").append('\
				<div id="sprite_' + s.id + '" class="spriteInfo">\
					<b>Name:</b> ' + s.name + '<br />\
					<b>Type:</b> ' + s.type + '<br />\
					<div id="excludeSprite_' + s.id + '" class="excludeSprite">[remove]</div>\
					<table border="0" cellpadding="0" cellspacing="5" width="100%">\
						<tr><td>x:</td>\
						<td><input type="text" id="sprite_' + s.id + '_x" class="smallEditorInput" value="' + s.x + '" />\
						<td>y:</td>\
						<td><input type="text" id="sprite_' + s.id + '_y" class="smallEditorInput" value="' + s.y + '" />\
						<td>w:</td>\
						<td><input type="text" id="sprite_' + s.id + '_width" class="smallEditorInput" value="' + w + '" />\
						<td>h:</td>\
						<td><input type="text" id="sprite_' + s.id + '_height" class="smallEditorInput" value="' + h + '" /></tr>\
					</table>\
					<table id="sprite_' + s.id + '_hints" border="0" cellpadding="0" cellspacing="5" width="100%"></table>\
					<table border="0" cellpadding="0" cellspacing="5" width="100%">\
						<tr><td>\
							<input type="button" id="updateSprite_' + s.id + '" class="updateSprite" value="Save Info" />\
							<input type="button" id="deleteSprite_' + s.id + '" class="deleteSprite" value="Delete" />\
						</td></tr>\
					</table>\
				</div>');

			// Now add in the editor hints
			for(var j in s.editorHints) {
				var hint = s.editorHints[j];
				var value = (typeof s[hint.field] != "undefined") ? s[hint.field] : hint.value;

				// Don't draw options for flags
				if(typeof hint.type != "undefined" && hint.type == "flag")
					continue;

				var label = (typeof hint.label != "undefined") ? hint.label : hint.field + ":";

				// Do specific work for the audio hint
				if(hint.type == "audio") {
					var input = '<select id="sprite_' + s.id + '_' + hint.field + '">';
					// Build up our known audio types and pre-select the sprites current value. In the case of non-audio sprites, this won't draw
					for (var k in this.knownAudio)
						input += '<option value="' + this.knownAudio[k] + '"' + (this.knownAudio[k] == s.audio ? ' selected="selected"' : '') + '>' + this.knownAudio[k] + '</option>';
					input += '</select>';
				}
				else if (hint.type == "animation") {
					var input = '<select id="sprite_' + s.id + '_' + hint.field + '">';
					for (var k in s.animation)
						input += '<option value="' + k + '"' + (value == k ? ' selected="selected"' : '') + '>' + k + '</option>';
					input += '</select>';
				}
				else {
					var input = '<input type="text" id="sprite_' + s.id + '_' + hint.field + '" class="mediumEditorInput" value="' + value + '" />';
				}

				$("#sprite_" + s.id + "_hints").append('\
					<tr><td>' + label + '</td>\
					<td>' + input + '</td></tr>');

			}

		};

		// Bind events to our selection's items
		$(".updateSprite").bind("click", function() {
			var id = $(this).attr("id").split("_")[1];
			var s = engine.getSprite(id);
			s.x = parseInt($("#sprite_" + id + "_x").val());
			s.y = parseInt($("#sprite_" + id + "_y").val());

			s.resize(
				parseInt($("#sprite_" + id + "_width").val()),
				parseInt($("#sprite_" + id + "_height").val())
			);

			// Now store the hints as described by the sprite
			for (var i in s.editorHints) {
				var hint = s.editorHints[i];
				s[hint.field] = $("#sprite_" + id + "_" + hint.field).val();
			}
		});
		$(".excludeSprite").bind("click", function() {
			var id = $(this).attr("id").split("_")[1];
			for(var i in self.currentSelection) {
				if(self.currentSelection[i].id == id) {
					// Remove from selection, remove from context, remove outline from render loop
					self.currentSelection.splice(i,1);
					self.updateContextTools();
					e.getSprite(id).selected = 0;
				}
			}
		});
		$(".deleteSprite").bind("click", function() {
			var id = $(this).attr("id").split("_")[1];
			for(var i in self.currentSelection) {
				if(self.currentSelection[i].id == id) {
					// Remove from selection, remove from context, DELETE from engine instantly
					self.currentSelection.splice(i,1);
					self.updateContextTools();
					engine.destroySprite(id, true);
				}
			}
		});
	};//end function updateContextTools

	/**
	 * Goes through each of the currentSelection items and updates their (x,y) positions
	 */
	this.moveSelection = function(x, y) {
		for (var i in this.currentSelection) {
			var s = this.currentSelection[i];
			// Update the (x,y) values in the selection
			s.x += x;
			s.y += y;
			// Update the values in the engine
			var engineSprite = engine.getSprite(s.id);
			engineSprite.x = s.x;
			engineSprite.y = s.y;
			// Update the values in the context tool
			$("#sprite_" + s.id + "_x").val(s.x);
			$("#sprite_" + s.id + "_y").val(s.y);
		}
	};//end function moveSelection

	/**
	 * Preview the level. Really?  This works?
	 */
	this.engineSpriteBackup = {};
	this.enginePreviewing = false;
	this.engineCameraBackupX = 0;
	this.engineCameraBackupY = 0;
	this.previewWorld = function() {
		if(!this.enginePreviewing) {
			// Back up the full sprite state.
			for(var layer in engine.sprites) {
				this.engineSpriteBackup[layer] = []
				for(var j in engine.sprites[layer]) {
					// Magic. This does a deep copy of the sprite object and stores it away into the backup object
					var clone = $.extend(true, {}, engine.sprites[layer][j]);
					this.engineSpriteBackup[layer].push(clone);
				}
			}

			// Remember the camera for when we come back
			this.engineCameraBackupX = engine.gs.cameraX;
			this.engineCameraBackupY = engine.gs.cameraY;
			engine.gs.cameraX = engine.getSprite("p1").x;
			engine.gs.cameraY = engine.getSprite("p1").y;

			// Alter the gamestate functionality
			engine.gs.showDebugging = false;
			engine.gs.mouseControl = false;
			engine.gs.keyControl = false;
			engine.gs.mousedownEvent = this.engineMousedown;
			engine.gs.mousemoveEvent = this.engineMousemove;
			engine.gs.mouseupEvent = this.engineMouseup;

			// Restore the original engine logic/render loops
			engine.logic = this.engineLogic;
			engine.render = this.engineRender;

			// Set the previewing flag!
			this.enginePreviewing = true;
			engine.debug(" *** Entered Level Preview Mode *** ");
		}
		else {
			// Hook the editor's logic/render back into the engine
			engine.logic = this.editorLogic;
			engine.render = this.editorRender;

			// Put the gamestate back into editor mode
			engine.gs.showDebugging = true;
			engine.gs.mouseControl = true;
			engine.gs.keyControl = true;
			engine.gs.mousedownEvent = this.mousedownEvent;
			engine.gs.mousemoveEvent = this.mousemoveEvent;
			engine.gs.mouseupEvent = this.mouseupEvent;

			engine.gs.cameraX = this.engineCameraBackupX;
			engine.gs.cameraY = this.engineCameraBackupY;

			// Pause all SFX
			for(var i in engine.audio)
				engine.audio[i].pause();

			// Start from scratch on the engine sprites. Clean them out!
			engine.sprites = {};

			// Restore the full backup of the sprites (then delete the backup)
			for(var layer in this.engineSpriteBackup) {
				engine.sprites[layer] = [];
				for(var j in this.engineSpriteBackup[layer]) {
					var clone = this.engineSpriteBackup[layer][j];
					engine.sprites[layer].push(clone);
				}
			}
			this.engineSpriteBackup = {};

			// We are no longer previewing a level. (phew!)
			this.enginePreviewing = false;
			engine.debug(" *** Exited Preview Mode *** ");
		}
	};//end function previewWorld

	/**
	 * Find sprites that overlap the (x,y) coordinate. Remember, we're in the engine namespace for this one :)
	 */
	engine.seekSprite = function(x, y) {
		var result = [];
		for (var i in this.sprites) {
			for (var j in this.sprites[i]) {
				var s = this.sprites[i][j];

				// We do want to clear sprites through inactive layers
				s.selected = false;

				// But we don't want to match through inactive layers!
				if(!self.drawLayers[i])
					continue;

				// Get the x, y, width, height of our test sprite
				var renderPoint = s.getRenderPoint(this.gs);
				var width = s.renderWidth;
				var height = s.renderHeight;

				// Compare and see if the click lands inside the bounding box
				if (x >= renderPoint.x && x <= renderPoint.x + width && y >= renderPoint.y && y <= renderPoint.y + height)
					result.push(s);
			}
		}
		return result;
	};//end function seekSprite

	// Keep a copy of the engine's logic and render function
	this.engineLogic = engine.logic;
	this.engineRender = engine.render;

	/**
	 * Override the logic function on the engine. NOTE: This will eventually be bound into the engine, so the context is misleading
	 * the "this" keyword will be referring to the engine object
	 */
	this.logicCalls = 0;
	this.logicLoop = this.logicStart = this.logicTime = null;
	this.editorLogic = function() {
		this.logicStart = new Date();
		this.logicCalls++;

		// Listening for event input every loop is a bit overkill ... so only listen every other loop
		if (this.logicCalls % 2 == 1) {
			// Move sprites to the left
			if (this.gs.keyLeft && self.currentSelection.length > 0)
				self.moveSelection(-1, 0);
			// Move sprites to the right
			if (this.gs.keyRight && self.currentSelection.length > 0)
				self.moveSelection(1, 0);
			// Move sprites up
			if (this.gs.keyUp && self.currentSelection.length > 0)
				self.moveSelection(0, -1);
			// Move sprites down
			if (this.gs.keyDown && self.currentSelection.length > 0)
				self.moveSelection(0, 1);
		}//if (listening for input events)

		this.logicTime = new Date() - this.logicStart;
		
		if(this.logicCalls == 1)
			$(this.canvas).css({
				background: this.sceneSettings.background
			});

		// Fire off the render step
		this.render();
	};//end function logic

	/**
	 * Override the render function on the engine. NOTE: This will eventually be bound into the engine, so the context is misleading
	 * the "this" keyword will be referring to the engine object
	 */
	this.renderTime = null;
	this.renderedSprites = 0;
	this.editorRender = function() {
		this.renderStart = new Date();
		this.renderedSprites = 0;

		// Clear the screen
		this.context.clearRect(0, 0, this.gs.width, this.gs.height);

		// Render each sprite, if applicable
		for (i in this.sprites) {

			// Don't draw disabled layers
			if(!self.drawLayers[i])
				continue;

			for (j in this.sprites[i]) {
				if (this.sprites[i][j] != null) {
					/**
					 * This is taken care of in chrome, but I suspect that Firefox doesn't know to avoid rendering objects which are too far off screen
					 */
					var s = this.sprites[i][j];
					var renderPoint = s.getRenderPoint(this.gs);

					// Bail if the sprite is too far off screen. Start testing right, then left (most likely), then bottom and top
					if(renderPoint.x > this.gs.width)
						continue;
					if(renderPoint.x + s.renderWidth < 0)
						continue;
					if(renderPoint.y > this.gs.height)
						continue;
					if(renderPoint.y + s.renderHeight < 0)
						continue;

					this.renderedSprites++;
					var suffix = (typeof this.sprites[i][j].currentAnimation == "string") ? "_" + this.sprites[i][j].currentAnimation : "";
					this.sprites[i][j].render(this.resources[this.sprites[i][j].name + suffix], this.context, this.gs, renderPoint);
					if(this.sprites[i][j].selected)
						this.sprites[i][j].outline(this.context, this.gs);
				}
			}
		}

		// Render the camera lines
		this.context.fillStyle = "rgba(0, 0, 255, 1.0)";
		this.context.fillRect(this.gs.cameraMinX - this.gs.cameraX - 1, 0, 1, this.gs.height);
		this.context.fillRect(this.gs.cameraMaxX - this.gs.cameraX + this.gs.width, 0, 1, this.gs.height);
		this.context.fillRect(0, this.gs.cameraMinY - this.gs.cameraY - 1, this.gs.width, 1);
		this.context.fillRect(0, this.gs.cameraMaxY - this.gs.cameraY + this.gs.height, this.gs.width, 1);

		// Render the mouse reticle
		this.context.fillStyle = "rgba(255, 255, 255, 0.5)";
		this.context.fillRect(this.gs.mouseX, 0, 1, this.gs.height);
		this.context.fillRect(0, this.gs.mouseY, this.gs.width, 1);

		// Render the FPS info
		var d = new Date();
		this.renderTime = d - this.renderStart;
		totalTime = d - this.logicStart;

		this.context.fillStyle = "rgba(255, 255, 255, 0.8)";
		this.context.fillRect(0, 0, this.gs.width, 15);

		this.context.fillStyle = "#000000";
		this.context.fillText("Total Sprites: " + this.countSprites(), 90, 10)
		this.context.fillText("Rendered: " + this.renderedSprites, 210, 10)
		this.context.fillText("Gravity: " + this.gs.gravity, 300, 10)
		this.context.fillText("FPS: " + Math.round(1000 / totalTime), 380, 10);
		this.context.fillText("Logic Time: " + this.logicTime + "ms", 440, 10);
		this.context.fillText("Render Time: " + this.renderTime + "ms", 540, 10);
		this.context.fillText("Total Frames: " + this.logicCalls, 740, 10);

		this.lastRenderTimestamp = new Date();
	};//end function render

	// Finally, attach the editor's logic and render functionality into the engine, replacing the original
	engine.logic = this.editorLogic;
	engine.render = this.editorRender;


	/**
	 * Attach the helper to the canvas
	 */
	$(function() {
		// Add the control box
		$("body").append($(document.createElement("div")).attr("id", "editorTools"));
		$("body").append($(document.createElement("div")).attr("id", "helperTools"));

		// Before we can draw a list of the sprites and such, we need to know which ones are available
		self.gatherKnownItems();

		// Build up some HTML for the create new sprite area
		var spriteOptions = "";
		for(var i in self.knownSprites)
			spriteOptions += '<option value="' + self.knownSprites[i] + '">' + self.knownSprites[i] + '</option>' + "\n";

		// Add the controls
		$("#editorTools").html('\
				<div id="mainHeader_left" class="mainEditorHeader">Main Tools</div>\
				<div id="mainEditorContent_left" class="mainEditorContent">\
					<div id="header_worldSettings" class="sectionHeader">World Settings</div>\
					<div id="section_worldSettings" class="editorSection">\
						<div style="font-size: 9px; text-align: center; font-weight: bold;">NO spaces, NO special characters!</div>\
						<table border="0" cellpadding="0" cellspacing="5" align="center"><tr>\
							<td>Scene Name</td>\
							<td><input id="sceneName" style="width: 102px;" value="' + engine.sceneSettings.name + '" /></td>\
						</tr><tr>\
							<td colspan="2" style="text-align: right;"><input type="button" id="sceneNameConfirm" value="Set Scene Name" /></td>\
						</tr></table>\
						<div style="font-size: 9px; text-align: center; font-weight: bold;">Camera mix/max values are enforced during gameplay.</div>\
						<table border="0" cellpadding="0" cellspacing="5" align="center"><tr>\
							<td>MinX:</td>\
							<td style="width: 50px;"><input id="cameraMinX" style="width: 45px;" value="' + engine.gs.cameraMinX + '" /></td>\
							<td>MinY:</td>\
							<td style="width: 50px;"><input id="cameraMinY" style="width: 45px;" value="' + engine.gs.cameraMinY + '" /></td>\
						</tr><tr>\
							<td>MaxX:</td>\
							<td><input id="cameraMaxX" style="width: 45px;" value="' + engine.gs.cameraMaxX + '" /></td>\
							<td>MaxY:</td>\
							<td><input id="cameraMaxY" style="width: 45px;" value="' + engine.gs.cameraMaxY + '" /></td>\
						</tr><tr>\
							<td colspan="4" style="text-align: right;"><input type="button" id="cameraBoundsConfirm" value="Set Camera Limits" /></td>\
						</tr></table>\
					</div>\
					<div id="header_createSprite" class="sectionHeader">Create new Sprite</div>\
					<div id="section_createSprite" class="editorSection">\
						<div style="font-size: 9px; text-align: center; font-weight: bold;">The sprite will draw to the center(ish) of the game screen.</div>\
						<table border="0" cellpadding="0" cellspacing="5" align="center">\
						<tr><td colspan="2"><select id="createSpriteType">' + spriteOptions + '</select></td></tr>\
						<tr><td><select id="createSpriteLayer">\
							<option value="foreground">Foreground</option>\
							<option value="game" selected="selected">Game Layer</option>\
							<option value="background1">Background 1</option>\
							<option value="background2">Background 2</option>\
							<option value="background3">Background 3</option>\
							<option value="background4">Background 4</option>\
							<option value="background5">Background 5</option>\
						</select></td>\
						<td><input type="button" id="createSprite" value="Create Sprite" /></td>\
						</tr></table>\
					</div>\
					<div id="header_updateCamera" class="sectionHeader">Camera Controls</div>\
					<div id="section_updateCamera" class="editorSection">\
						<table border="0" cellpadding="0" cellspacing="5" width="100%">\
							<tr><td>x:</td>\
							<td><input type="text" id="updateCamera_x" class="smallEditorInput" value="0" /></td>\
							<td>y:</td>\
							<td><input type="text" id="updateCamera_y" class="smallEditorInput" value="0" /></td>\
							<td><input type="button" id="updateCameraButton" style="margin-left: 10px;" value="Move Camera" /></td></tr>\
						</table>\
					</div>\
					<div id="header_toggleContexts" class="sectionHeader">Toggle Layers</div>\
					<div id="section_toggleContexts" class="editorSection"><table border="0" cellpadding="0" cellspacing="5" style="width: 100%;">\
						<tr><td style="width: 50%;"><input type="checkbox" id="context_background5" class="toggleContext" checked="checked" /> Background #5</td></tr>\
						<tr><td style="width: 50%;"><input type="checkbox" id="context_background4" class="toggleContext" checked="checked" /> Background #4</td>\
						<td style="width: 50%;"><input type="checkbox" id="context_background3" class="toggleContext" checked="checked" /> Background #3</td></tr>\
						<tr><td><input type="checkbox" id="context_background2" class="toggleContext" checked="checked" /> Background #2</td>\
						<td><input type="checkbox" id="context_background1" class="toggleContext" checked="checked" /> Background #1</td></tr>\
						<tr><td><input type="checkbox" id="context_game" class="toggleContext" checked="checked" /> Game Plane</td>\
						<td><input type="checkbox" id="context_foreground" class="toggleContext" checked="checked" /> Foreground</td></tr></table>\
					</div>\
					<div id="header_editorButtons" class="sectionHeader">Useful Buttons</div>\
					<div id="section_editorButtons" class="editorSection" style="padding: 5px 0; text-align: center;">\
						<input type="button" id="getState" value="Dump State" /><input type="button" id="toggleDebug" value="Show Invisible" /><input type="button" id="previewWorld" value="Preview World" /><br />\
						<input type="button" id="generateWorld" value="Generate World" style="font-size: 22px;" />\
					</div>\
				</div>');

		$("#helperTools").html('\
				<div id="mainHeader_right" class="mainEditorHeader">Context Tools</div>\
				<div id="mainEditorContent_right" class="mainEditorContent">\
					<div id="header_selectionInfo" class="sectionHeader">Selection Information</div>\
					<div id="section_selectionInfo" class="editorSection">\
						<i style="color: #AAA;">No sprites selected</i>\
					</div>\
				</div>');

		$("#sceneNameConfirm").bind("click", function() {
			engine.sceneSettings.name = $("#sceneName").val();
		});

		$("#cameraBoundsConfirm").bind("click", function() {
			engine.gs.cameraMinX = $("#cameraMinX").val();
			engine.gs.cameraMinY = $("#cameraMinY").val();
			engine.gs.cameraMaxX = $("#cameraMaxX").val();
			engine.gs.cameraMaxY = $("#cameraMaxY").val();
		});

		// Bind the controls
		$(".mainEditorHeader").bind("click", function() {
			var id = $(this).attr("id").split("_")[1];
			$("#mainEditorContent_" + id).css({
				display: ($("#mainEditorContent_" + id).css("display") == "none") ? "block" : "none"
			});
		});

		$(".sectionHeader").bind("click", function() {
			var id = $(this).attr("id").split("_")[1];
			$("#section_" + id).css({
				display: ($("#section_" + id).css("display") == "none") ? "block" : "none"
			});
		});

		$("#createSprite").bind("click", function() {
			var type = $("#createSpriteType").val();
			var layer = $("#createSpriteLayer").val();

			// Chop the x depending on the layer
			var chop = 1;
			if (layer == "background4") chop = 1/8;
			else if (layer == "background3") chop = 1/4;
			else if (layer == "background2") chop = 1/2;

			var x = (engine.gs.cameraX * chop) + (engine.gs.width / 2); 
			var y = (engine.gs.cameraY * chop) + (engine.gs.height / 2);

			engine.declareSprite(layer, new window[type]({
				x: x + (Math.floor(Math.random() * 41) - 20),
				y: y + (Math.floor(Math.random() * 41) - 20)
			}));
		});

		$("#updateCameraButton").bind("click", function() {
			engine.gs.cameraX = parseInt($("#updateCamera_x").val());
			engine.gs.cameraY = parseInt($("#updateCamera_y").val());
		});

		$(".toggleContext").bind("click", function() {
			var id = $(this).attr("id").split("_")[1];
			self.drawLayers[id] = !self.drawLayers[id];
		});

		$("#getState").bind("click", function(){
			engine.debugging = true;
			engine.state();
			$(this).blur();
		});
		$("#toggleDebug").bind("click", function(){
			engine.gs.showDebugging = !engine.gs.showDebugging;
			$(this).blur();
		});
		$("#previewWorld").bind("click", function() {
			self.previewWorld();
		});

		// World generation script!
		$("#generateWorld").bind("click", function() {
			self.generate();
		});
	});

	/**
	 * Generate the world!
	 */
	this.generate = function() {
		var js = 'function ' + engine.sceneSettings.name + '() {' + "\n\n";
		js += "\t" + 'p = {' + "\n";
		js += "\t\t" + 'name: "' + engine.sceneSettings.name + '",' + "\n";
		js += "\t\t" + 'background: "#FFFFFF url(images/' + engine.sceneSettings.name + '.jpg) no-repeat",' + "\n";
		js += "\t\t" + 'cameraMinX: ' + engine.gs.cameraMinX + ',' + "\n";
		js += "\t\t" + 'cameraMinY: ' + engine.gs.cameraMinY + ',' + "\n";
		js += "\t\t" + 'cameraMaxX: ' + engine.gs.cameraMaxX + ',' + "\n";
		js += "\t\t" + 'cameraMaxY: ' + engine.gs.cameraMaxY + "\n";
		js += "\t" + '};' + "\n";

		js += "\n\t" + 'this.scene = new scene(p);' + "\n\n";

		for (var i in engine.sprites) {
			for (var j in engine.sprites[i]) {
				// Makes the sprite much easier to refer to later
				var s = engine.sprites[i][j];

				js +=
'	this.scene.addSprite("' + i + '", new sprite_' + s.name + '(';

				var params = [];

				if (typeof s.x == "number" && typeof s.y == "number") {
					params.push("x: " + s.x);
					params.push("y: " + s.y);
				}

				// Write out the hints and other editor flags
				for(var k in s.editorHints) {
					var hint = s.editorHints[k];

					// Handle generateSize hints
					if(hint.field == "generateSize") {
						params.push("width: " + s.renderWidth);
						params.push("height: " + s.renderHeight);
					}
					// Handle audio hints
					else if(hint.field == "audio"
					&& (typeof s.audio != "undefined" && s.audio != "")) {
						params.push('audio: "' + s.audio + '"');
					}
					// Handle all other hints
					else {
						// Don't bother writing out the defaults - if nothing has been stored OR changed then the hint doesn't matter
						if(typeof s[hint.field] == "undefined" || s[hint.field] == hint.value)
							continue;
						// Otherwise, push the hint to the params field
						params.push(hint.field + ': "' + s[hint.field] + '"');
					}
				}

				// If we have some parameters to write, do so now
				if(params.length > 0)
					js += "{\n\t\t" + params.join(",\n\t\t") + "\n\t}";

				js += '));' + "\n";
			}
		}
		js +=
'	return this.scene;' + "\n" +
'}//end AUTO-GENERATED scene';

		// Now that we have the code, write it out
		$("body").append('<pre>' + js + '</pre>');
	};//end function generate

}//end object

window.editor = new editor();

}//end EDIT MODE query

})(window, e);