/**
 * The gamestate is a set of parameters for controlling the world settings.
 * @author Torgie
 */
function gameState(canvas) {

	// Set the canvas width and height
	this.width = $(canvas).innerWidth();
	this.height = $(canvas).innerHeight();

	// Used for "hammer time" effects, simply scales the motion calculations
	this.originalWorldSpeed = 1; // Original worldSpeed setting
	this.worldSpeedMinimum = 0.02; // What to set the worldSpeed to during engine.hammerTime()
	this.worldSpeedRamp = 1.02; // Exponent for ramping back into normal time
	this.originalGravity = 0.50; // Original gravity setting

	this.worldSpeed = this.originalWorldSpeed;
	this.gravity = this.originalGravity;

	// Higher bounciness means less speed lost during a bounce. max: n/a, bypass: 1, min: n/a
	this.wallBounciness = .5;

	// Target framerate, 1000 / FPS = millis
	this.fps = 1000 / 30;

	// Whether or not to draw the FPS information
	this.showDebugging = false;

	this.showMeter = false;
	this.fuel = 250;
	
	this.playerX = 0;
	this.playerY = 0;

	this.cameraX = 0;
	this.cameraY = 0;

	this.cameraShake = 0;
	this.cameraShakeRamp = 1.1;

	// The mouse location in the engine canvas
	this.mouseControl = false;
	this.keyControl = false;

	this.clearKeys = function() {
		this.keyLeft = false;
		this.keyUp = false;
		this.keyRight = false;
		this.keyDown = false;
		this.keyA = false;
		this.keyD = false;
		this.keyZ = false;
		this.keyX = false;
		this.keySpace = false;
		
	};
	this.clearKeys();

	// Maps a key event to internal flags
	this.mapKeyEvent = function(pressed, event) {
		if(!this.keyControl)
			return true;

		switch (event.keyCode) {
			case 37:
				if(this.keyLeft && pressed)
					return;
				this.keyLeft = pressed;
				break;
			case 90:
				if(this.keyLeft && pressed)
					return;
				this.keyLeft = pressed;
				break;
			case 38:
				if(this.keyUp && pressed)
					return;
				this.keyUp = pressed;
				break;
			case 39:
				if(this.keyRight && pressed)
					return;
				this.keyRight = pressed;
				break;
			case 88:
				if(this.keyRight && pressed)
					return;
				this.keyRight = pressed;
				break;
			case 40:
				if(this.keyDown && pressed)
					return;
				this.keyDown = pressed;
				break;
			case 32:
				if(this.keySpace && pressed)
					return;
				this.keySpace = pressed;
				break;
			case 65:
				if(this.keyA && pressed)
					return;
				this.keyA = pressed;
				break;
			case 68:
				if(this.keyD && pressed)
					return;
				this.keyD = pressed;
				break;
			default:
				// Unmapped key, return
				return true;
		}//switch
	};//end function mapKeyEvent

	this.mapMouseEvent = function(event) {
		if(!this.mouseControl)
			return true;

		switch (event.type) {
			case "mousedown":
			case "mousemove":
			case "mouseup":
				// This will fire the callback event, mousedownEvent, etc.
				return this[event.type + "Event"](event);
				break;
			default:
				return true;
		}//switch
	};//end function mapMouseEvent

	// Map mouse activity to functions
	this.mousedownEvent = function(event) { return true; };
	this.mouseupEvent = function(event) { return true; };
	this.mousemoveEvent = function(event) { return true; };

 }//end object gameState