<?php
	// Deal with uploaded sprites
	if(isset($_FILES['sprite'])) {
		echo renderSpriteEditor(handleUpload());
		die();
	}

	echo renderEmpty();

	/**
	 * Renders an empty prompt to upload a sprite
	 */
	function renderEmpty() {
		ob_start();
?>
	<?=_header()?>

	<form method="post" action="spriteMaker.php" enctype="multipart/form-data">
		<input type="hidden" name="MAX_FILE_SIZE" value="<?=(1024*100)?>" />
		<h1>Upload your sprite animations</h1>

		<div style="margin-left: 20px;">

			<div id="inputSource" style="margin-top: 10px;">
				<div class="inputLabel" style="font-size: 12px;">Name ("run", "idle", etc)</div>
				<input type="text" name="spriteName[]" maxlength="100" style="width: 150px;" />
				<input type="file" name="sprite[]" />
	
			</div>

			<div style="width: 300px; text-align: right; margin-top: 20px;">
				<div>
					<span id="newSprite" class="fakeLink">Add another file.</span>
					<input type="submit" value="Upload Sprites" />
				</div>
				<span style="font-size: 11px;">Files without names, or names without files will be ignored.</span>
			</div>

		</div>

	</form>

<script type="text/javascript">
//<![CDATA[
	$("#newSprite").bind("click", function() {
		var source = $("#inputSource");
		var clone = source.clone(true);
		clone.attr("id", "");
		source.after(clone);
	});
//]]>
</script>
	<?=_footer()?>
<?php
		return ob_get_clean();
	}


	/**
	 * Renders the sprite editor HTML and returns it
	 */
	function renderSpriteEditor($files) {
		ob_start();
?>
	<?=_header()?>

	<h1>Sprite Preview <span id="closePreview" class="fakeLink" style="padding-left: 30px; font-size: 12px; color: #FFF;">Click to toggle</span></h1>

	<div id="previewBox">
		<div style="width: 100%; overflow-x: scroll">
			<table>
<?php
		$first = TRUE;
		foreach($files as $name => $info) {
			if($first) {
				list($pre, $post) = split('_', $info['file'], 2);
				if(strpos($post, '_') !== FALSE)
					$post = substr($post, 0, strpos($post, '_'));
				else
					$post = substr($post, 0, strpos($post, '.'));
				$preFillSource = $pre . '_' . $post;
				$first = FALSE;
			}
?>
				<tr>
					<td><b><?=$name?></b><br /><?=$info['file']?></td>
					<td><img src="<?=$info['src']?>" alt="<?=$info['file']?>" style="border: 1px #aaa dashed;" /></td>
				</tr>
				<?php
						unset($files[$name]['src']);
					}
				?>
			</table>
		</div>
	</div>

	<div style="height: 20px;"></div>

	<div style="min-width: 600px;">
		<div style="float: left; width: 50%; min-width: 300px;">
			<h1>Basic Settings</h1>
			<table border="0" cellpadding="0" cellspacing="5" style="margin: 0 10px;">
				<tr>
					<td><b>World Number</b></td>
					<td><select id="spriteWorld">
						<option value="0">World 0</option>
						<option value="1" selected="selected">World 1</option>
						<option value="2">World 2</option>
						<option value="3">World 3</option>
						<option value="4">World 4</option>
						<option value="5">World 5</option>
					</select></td>
				</tr>
				<tr>
					<td><b>Sprite Name</b></td>
					<td><input type="text" id="spriteName" maxlength="100" value="<?=$preFillSource?>" /></td>
				</tr>
			</table>
		</div>
		<div style="float: left; width: 50%; min-width: 300px;">
			<h1>Behaviors</h1>
			<table border="0" cellpadding="0" cellspacing="5" style="margin: 0 10px;">
				<tr>
					<td><b>Sprite Type</b></td>
					<td><select id="spriteType">
						<option value="scenery">Scenery</option>
						<option value="ground">Ground</option>
						<option value="enemy">Enemy</option>
					</select></td>
				</tr>
				<tr>
					<td><b>Animates?</b></td>
					<td><select id="spriteAnimates">
						<option value="0">No</option>
						<option value="1">Yes</option>
					</select></td>
				</tr>
			</table>
		</div>
		<div style="clear: both;"></div>
	</div>

	<div style="height: 20px;"></div>
	<h1>Javascript Code</h1>
	<div id="outputBox" style="padding: 10px; font: normal 12px monospace; white-space: pre-wrap;">Your Javascript will appear here when changes are made.</div>

<script type="text/javascript">
//<![CDATA[

	var sourceData = eval("(" + '<?=json_encode($files)?>' + ")");

	$("#closePreview").bind("click", function() {
		if($("#previewBox").css("display") == "none")
			$("#previewBox").css({ display: "block" });
		else
			$("#previewBox").css({ display: "none" });
	});

	$("#spriteName").bind("keyup", function() {
		var val = $(this).val().replace(/[^a-zA-Z0-9_]/g, "");
		$(this).val(val);
		updateJavascript();
	});

	$("#spriteAnimates").bind("change", function() {
		updateJavascript();
	});

	$("#spriteType").bind("change", function() {
		updateJavascript();
	});

	function updateJavascript() {
		var box = $("#outputBox");
		box.empty();
		box.append('function sprite_' + $("#spriteName").val() + '(p) {' + "\n");
		box.append('	p.type = "' + $("#spriteType").val() + '";' + "\n");
		box.append('	p.name = "' + $("#spriteName").val() + '";' + "\n");

		if ($("#spriteAnimates").val() == 0) {
			for (var name in sourceData) {
				var src = sourceData[name];
				box.append('	p.src = "images/sprites/world' + $("#spriteWorld").val() + '/' + src.file + '";' + "\n");
				box.append('	p.width = ' + src.width + ';' + "\n");
				box.append('	p.height = ' + src.height + ';' + "\n");
				break;
			}
		}
		else {
			box.append('	p.src = {' + "\n");
			for (var name in sourceData) {
				var src = sourceData[name];
				box.append('		"' + name + '": "images/sprites/world' + $("#spriteWorld").val() + '/' + src.file + '",' + "\n");
			};
			box.append('	};' + "\n");
		}

		box.append("\n");
		box.append('	this.sprite = new sprite(p);' + "\n");

		if ($("#spriteType").val() == "ground") {
				box.append("\n");
				box.append('	this.sprite.collideOffsetX = 0;' + "\n");
				box.append('	this.sprite.collideOffsetY = 0;' + "\n");
				box.append('	this.sprite.collideWidth = this.sprite.width;' + "\n");
				box.append('	this.sprite.collideHeight = this.sprite.height;' + "\n");
		}

		box.append("\n");
		box.append('	return this.sprite;' + "\n");
		box.append('}');
	}
	updateJavascript();

//]]>
</script>
	<?=_footer()?>
<?php
		return ob_get_clean();
	}




	function handleUpload() {
		$files = $_FILES['sprite'];

		// Test for errors
		foreach($files['error'] as $code)
			if($code !== 0)
				die("\n" . 'Error uploading files.' . pa($files));

		// Store the PNGs directly into memory
		$return = array();
		foreach($files['tmp_name'] as $index => $tmp) {
			$size = getimagesize($tmp);
			$return[$_POST['spriteName'][$index]] = array(
				'src' => 'data:image/png;base64,' . base64_encode(file_get_contents($tmp)),
				'file' => str_replace('"', '', $files['name'][$index]),
				'width' => $size[0],
				'height' => $size[1]
			);
		}

		return $return;
	}

	function _header() {
		ob_start();
?>
<!DOCTYPE html>
<html>
<head>
	<meta charset="utf-8">

	<title>Sprite Editor</title>

	<style type="text/css">
body {
	margin: 0;
	padding: 0;
	font: normal 13px "Trebuchet MS", Verdana, Geneva, Arial, Helvetica, sans-serif;
}
h1 {
	margin: 0;
	padding: 5px 10px;
	background: #0055B4;
	color: #FFF;
}
.fakeLink {
	text-decoration: underline;
	color: #0055B4;
	cursor: pointer;
}
	</style>
	<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jquery/1.4.3/jquery.min.js"></script>
</head>
<body>
<?php
		return ob_get_clean();
	}//end function header

	function _footer() {
		ob_start();
?>
</body>
</html>
<?
		return ob_get_clean();
	}//end function footer

	function pa($item) {
		echo '<pre>';
		print_r($item);
		echo '</pre>';
	}//end function pa

?>