/**
 * @author Torgie
 */

/**
 * Hands control over to the player
 */
function sprite_gp_givePlayerControl(p) {
	p.name = "gp_givePlayerControl";
	p.color = "#00FFFF";

	this.sprite = new sprite_invisible(p);

	this.sprite.collide = function(engine, hit) {
		switch (hit.type) {
			case "player":
				engine.gs.clearKeys();
				engine.gs.keyControl = true;
				engine.destroySprite(this.id);
				break;
		}
	};


	return this.sprite;
}

/**
 * Represents a generic collision object. Must be set with a width/height on creation to use
 */
function sprite_gp_collision(p) {
	p.name = "gp_collision";
	p.color = "#FF00FF";

	this.sprite = new sprite_invisible(p);
	return this.sprite;
}

/**
 * Plays a sound effect of some sort when the player collides with it.
 * The exact effect to be played is described by the instancing of this sprite.
 * Like gp_collision, needs a width/height to be used
 */
function sprite_gp_sfx(p) {
	p.name = "gp_sfx";
	p.color = "#FFFF44";

	this.sprite = new sprite_invisible(p);
	this.sprite.audio = (typeof p.audio == "string") ? p.audio : "";

	this.sprite.editorHints.push({
		field: "audio",
		type: "audio"
	});

	this.sprite.collide = function(engine, hit) {
		switch (hit.type) {
			case "player":
				engine.playAudio(p.audio)
				engine.destroySprite(this.id, true);
				break;
		}
	};

	return this.sprite;
}

/**
 * Lodges a checkpoint to be created within the engine
 */
function sprite_gp_checkpoint(p) {
	p.name = "gp_checkpoint";
	p.color = "#FFFFFF";
	p.width = 20;
	p.height = 400;
	
	this.sprite = new sprite_invisible(p);

	this.sprite.collide = function(engine, hit) {
		switch (hit.type) {
			case "player":
				engine.checkpoint();
				engine.destroySprite(this.id, true);
				break;
		}
	};

	return this.sprite;
}