/**
 * @author Torgie
 */
function sprite_fg_heart() {
	p.type = "ui";
	p.name = "fg_heart";
	p.src = "images/sprites/Heart.png";
	p.width = 30;
	p.height = 29;
	p.uniqueID = "playerHealth";

	this.sprite = new sprite(p);

	// 2 hits and we're dead!
	this.sprite.hits = 0;

	this.sprite.render = function(resource, context, gs, renderPoint) {

		var sourceX = 30 * this.hits;

		var x = gs.width - 40;
		var y = 10;

		// Draw the sprite to the screen
		try {
			context.drawImage(
				resource,			// Resource object
				sourceX,			// Source X position
				0,					// Source Y position
				this.width,			// Source width
				this.height,		// Source height
				x,					// Target X position
				y,					// Target Y position
				this.renderWidth,	// Target width
				this.renderHeight	// Target height
			);
		}
		catch(e) {
			console.debug(this.name + ".drawImage(" + resource + ",0,0," + this.width + "," + this.height + "," + x + "," + y + "," + this.width + "," + this.height + ")");
		}
	}

	return this.sprite;
}

function sprite_gp_player(p) {
	p.type = "player";
	p.name = "gp_player";
	p.src = {
		idle:  "images/sprites/RGB_idle.png",
		run:   "images/sprites/RGB_run.png",
		jump:  "images/sprites/RGB_jump.png",
		fall:  "images/sprites/RGB_fall.png",
		worry: "images/sprites/RGB_worry.png",
		speak: "images/sprites/RGB_speak.png",
		hide: "images/sprites/RGB_hide.png"
	};

	// This is the width and height of the source sprite
	p.width = 72;
	p.height = 75;

	// Collision box
	p.collideOffsetX = 20;
	p.collideOffsetY = 15;
	p.collideWidth = 32;
	p.collideHeight = 60;

	// Generate the sprite
	this.sprite = new sprite(p);

	this.sprite.currentAnimation = (typeof p.currentAnimation != "undefined") ? p.currentAnimation : "idle";

	this.sprite.editorHints.push({
		field: "currentAnimation",
		type: "animation",
		label: "Animation:",
		value: "idle"
	});

	this.sprite.animation = {
		idle: {
			frames: 4,
			currentFrame: 0,
			flipEvery: 6,
			flip: 0
		},
		run: {
			frames: 4,
			currentFrame: 0,
			flipEvery: 15,
			flip: 0
		},
		jump: {
			frames: 2,
			currentFrame: 0,
			flipEvery: 15,
			flip: 0
		},
		fall: {
			frames: 3,
			currentFrame: 0,
			flipEvery: 15,
			flip: 0
		},
		worry: {
			frames: 6,
			currentFrame: 0,
			flipEvery: 2,
			flip: 0
		},
		speak: {
			frames: 4,
			currentFrame: 0,
			flipEvery: 5,
			flip: 0
		},
		hide: {
			frames: 1,
			currentFrame: 0,
			flipEvery: 1,
			flip: 0
		}
	};

	this.sprite.audio = {
		step1: {
			src: [
				"sfx/step1.ogg",
				"sfx/step1.wav"
			]
		},
		step2: {
			src: [
				"sfx/step2.ogg",
				"sfx/step2.wav"
			]
		},
		step3: {
			src: [
				"sfx/step3.ogg",
				"sfx/step3.wav"
			]
		},
		step4: {
			src: [
				"sfx/step4.ogg",
				"sfx/step4.wav"
			]
		},
		step5: {
			src: [
				"sfx/step5.ogg",
				"sfx/step5.wav"
			]
		}
	};

	this.sprite.fallForever = (typeof p.fallForever == "boolean") ? p.fallForever : false;

	// Add some custom fields to the sprite
	this.sprite.friction = (1 - 0.15);
	this.sprite.acceleration = .5;
	this.sprite.airTurnPenalty = 2; // acceleration divided by penalty when jumping

	// Are we jumping?
	this.sprite.jumping = true;
	// If we are, how fast do we jump?
	this.sprite.jumpImpulse = -9.5;

	this.sprite.facingRight = true;

	this.sprite.invul = false;
	this.sprite.invulCounter = 0;

	// Maximum speed limits
	this.sprite.maxSpeedX = 8;
	this.sprite.maxSpeedY = 10;

	// Controlling the camera?
	this.sprite.controlCamera = true;

	this.sprite.destroy = function(engine, i, j) {
		if(this.invul)
			return;

		var heart = engine.getSprite("playerHealth");

		if (heart.hits == 0) {
			heart.hits = 1;
			engine.declareSprite("foreground", new sprite_redSplode({
				x: this.x + (this.width / 2) - 50,
				y: this.y
			}));
			/**
			 * POW!  We got hit!
			 */
			this.invul = true;
			return;
		}

		if (heart.hits == 1) {
			heart.hits = 0;
			engine.declareSprite("foreground", new sprite_redSplode({
				x: this.x + (this.width / 2) - 50,
				y: this.y
			}));
			setTimeout(function() {
				//if for some reason the player is killed while control is turned off, turn it back on before restoring the checkpoint
				if (engine.gs.keyControl == false) {
					engine.gs.keyControl = true;
				}
				engine.restoreCheckpoint();
			}, 500);
		}
			
	};

	// Declare the logic step
	this.sprite.logic = function(engine) {
		// Jump!
		if (!this.jumping && engine.gs.keySpace && this.speedY == 0) {
			this.speedY = this.jumpImpulse;
			this.jumping = true;
			this.setAnimation("jump");

			// Pick a footstep between 1 and 5
			sB = engine.getSprite("sfxButton");
			if(sB.currentAnimation == "on")
			engine.playAudio("gp_player_step" + (Math.floor(Math.random() * 5) + 1));

			engine.declareSprite("game", new sprite_tinyCloud({
				x: this.facingRight ? this.x : this.x + this.width,
				y: this.y + this.height - 16,
				speedX: this.facingRight ? -4 : 4
			}));
		}

		// Move right
		if (engine.gs.keyRight) {
			// If in the air, move more slowly
			if(Math.abs(this.speedY) > .01)
				this.speedX += (this.acceleration / this.airTurnPenalty)
			else
				this.speedX += this.acceleration;

			// Facing which direction
			if (!this.jumping) {
				this.facingRight = true;
				this.setAnimation("run");
			}
		}// right arrow

		// Move left
		if (engine.gs.keyLeft) {
			// If in the air, move more slowly
			if(Math.abs(this.speedY) > .01)
				this.speedX -= (this.acceleration / this.airTurnPenalty);
			else
				this.speedX -= this.acceleration;

			// Facing which direction
			if (!this.jumping) {
				this.facingRight = false;
				this.setAnimation("run");
			}
		}// left arrow

		// Friction is nerdy
		if(!this.jumping && !engine.gs.keyRight && !engine.gs.keyLeft)
			this.speedX *= this.friction;

		// Returning to idle?
		if (Math.abs(this.speedX) < .25 && this.currentAnimation == "run" && !engine.gs.keyRight && !engine.gs.keyLeft && engine.gs.keyControl) {
			this.setAnimation("idle");
		}

		// Falling?
		if (this.speedY > 4 && this.currentAnimation != "fall" && engine.gs.keyControl) {
			this.setAnimation("fall");
			this.jumping = true;
		}


		// Gravity is nerdy, too
		this.speedY += engine.gs.originalGravity;

		// Cap left speed
		if(engine.gs.keyRight && this.speedX > this.maxSpeedX)
			this.speedX = this.maxSpeedX;
		// Cap right speed
		else if (engine.gs.keyLeft && this.speedX < (this.maxSpeedX * -1))
			this.speedX = (this.maxSpeedX * -1);
		// Stop if slow enough
		else if (Math.abs(this.speedX) < .01)
			this.speedX = 0;

		// Don't go down too fast
		if(this.speedY > this.maxSpeedY)
			this.speedY = this.maxSpeedY;
		// Stop if slow enough
		else if (Math.abs(this.speedY) < .01)
			this.speedY = 0;

		// Update the positions
		this.x += this.speedX;
		if (this.x + this.collideOffsetX < 0) {
			this.x = 0 - this.collideOffsetX;
			this.speedX *= -0.2;
		}

		this.y += this.speedY;
		if(this.y > engine.gs.height && !this.fallForever) {
			//if for some reason the player falls to death while control is turned off, turn it back on before restoring the checkpoint
			if (engine.gs.keyControl == false) {
				engine.gs.keyControl = true;
			}
			engine.restoreCheckpoint();
		}
		// Move the camera relative to the player
		var cameraX = this.x - 200;
		var cameraY = this.y - 250;

		if(this.controlCamera)
			engine.updateCamera(cameraX, cameraY);

		// Update the frameFlip contribution
		var animation = this.animation[this.currentAnimation];

		if(this.currentAnimation == "idle" || this.currentAnimation == "worry" || this.currentAnimation == "speak")
			animation.flip += 1;
		else if(this.currentAnimation == "run")
			animation.flip += Math.abs(this.speedX);
		else if(this.currentAnimation == "jump" || this.currentAnimation == "fall")
			animation.flip += Math.abs(this.speedY);

		if (this.invul)
			this.invulCounter++;

		if (this.invulCounter > 45) {
			this.invul = false;
			this.invulCounter = 0;
		}

	};//end function logic

	this.sprite.collide = function(engine, hit) {

		var l1 = this.x + this.collideOffsetX;
		var t1 = this.y + this.collideOffsetY;
		var r1 = l1     + this.collideWidth;
		var b1 = t1     + this.collideHeight;

		var l2 = hit.x + hit.collideOffsetX;
		var t2 = hit.y + hit.collideOffsetY;
		var r2 = l2    + hit.collideWidth;
		var b2 = t2    + hit.collideHeight;

		// If we were were about to land on the sprite...
		if (b1 - this.speedY <= t2) {
			switch (hit.type) {
				case "ground":
				case "object":
					// Reset the object up on top of the block
					this.y = t2 - this.height;
					this.speedY = 0;
					if (this.jumping) {
						this.setAnimation((Math.abs(this.speedX) > .25) ? "run" : "idle");
						this.jumping = false;
					}
					break;
				case "haunt":
					engine.destroySprite(this.id)
					break;
				case "spikes":
					// Reset the object up on top of the block
					this.y = t2 - this.height;
					this.speedY = 0;
					if (this.jumping) {
						this.setAnimation((Math.abs(this.speedX) > .25) ? "run" : "idle");
						this.jumping = false;
					}
					engine.destroySprite(this.id)
					break;
				case "enemy":
					this.speedY = -10;
					engine.destroySprite(hit.id);
					engine.declareSprite("foreground", new sprite_splode({
						x: this.x + (this.width / 2) - 50,
						y: this.y
					}));
					break;
			}
		}
		// If we hit the block from the left
		else if (r1 - this.speedX <= l2) {
			switch (hit.type) {
				case "haunt":
				case "enemy":
					engine.destroySprite(this.id)
					break;
				case "ground":
				case "spikes":
				case "object":
					this.x = l2 - this.width + this.collideOffsetX;
					this.speedX = 0;
					break;
			}
		}

		// If we hit the block from the right
		else if (l1 - this.speedX >= r2) {
			switch (hit.type) {
				case "haunt":
				case "enemy":
					engine.destroySprite(this.id)
					break;
				case "ground":
				case "spikes":
				case "object":
					this.x = r2 - this.collideOffsetX;
					this.speedX = 0;
					break;
			}
		}

		// If we were about to push the sprite from below
		else if (t1 - this.speedY >= b2) {
			this.y = b2 - this.collideOffsetY;
			this.speedY = 0;
			switch (hit.type) {
				case "haunt":
				case "enemy":
					engine.destroySprite(this.id)
					break;
				case "object":
					engine.destroySprite(hit.id);
					break;
			}
		}

		engine.gs.playerX = this.x;
		engine.gs.playerY = this.y;
	};//end function collide

	this.sprite.debugRender = function(resource, context, gs, renderPoint) {
		context.fillStyle = "rgba(255, 255, 255, 0.5)";
		context.fillRect(
			renderPoint.x + this.collideOffsetX,
			renderPoint.y + this.collideOffsetY,
			this.collideWidth,
			this.collideHeight
		);
	};//end function render

	// This is the fancy part - return our internal sprite
	return this.sprite;
}//end object