function sprite_gp_rous(p) {
	p.type = "enemy";
	p.name = "gp_rous";
	p.src = {
		run: "images/sprites/world1/gp_rous_run.png",
		die: "images/sprites/world1/gp_rous_die.png"
	};
	p.width = 165;
	p.height = 34;

	this.sprite = new sprite(p);

	this.sprite.collideWidth = 63;
	this.sprite.collideHeight = 28;
	this.sprite.collideOffsetX = 52;
	this.sprite.collideOffsetY = 6;

	this.sprite.currentAnimation = "run";
	this.sprite.animation = {
		run: {
			frames: 4,
			currentFrame: 0,
			flipEvery: 20,
			flip: 0
		},
		die: {
			frames: 2,
			currentFrame: 0,
			flipEvery: 4,
			flip: 0
		}
	};

	this.sprite.friction = (1 - 0.1);
	this.sprite.acceleration = .15;

	this.sprite.facingRight = false;
	this.sprite.maxSpeedX = 4;
	this.sprite.maxSpeedY = 10;

	// How many frames to draw the death animation until destroying the sprite
	this.sprite.deathFrames = 0;

	this.sprite.destroy = function(engine, i, j) {
		this.setAnimation("die");
		this.type = "dead";
	};

	// Declare the logic step
	this.sprite.logic = function(engine) {
		if(this.currentAnimation == "die") {
			this.animation.die.flip += 1;
			if(this.deathFrames++ >= 36)
				engine.destroySprite(this.id, true);
			return;
		}
		else if(this.currentAnimation == "idle")
			this.animation.idle.flip += 1;
		else
			this.animation[this.currentAnimation].flip += (Math.abs(this.speedX) * engine.gs.worldSpeed);

		// Attack the player if the player is within 500 px
		if(this.facingRight)
			this.speedX += this.acceleration;
		else 
			this.speedX -= this.acceleration;

		this.speedY += engine.gs.gravity;

		// Don't run to fast!
		if (this.speedX < 0 && Math.abs(this.speedX) > this.maxSpeedX)
			this.speedX = (this.maxSpeedX * -1);
		else if (this.speedX > this.maxSpeedX)
			this.speedX = this.maxSpeedX;
		else if (Math.abs(this.speedX) < .01)
			this.speedX = 0;

		// Don't go down too fast
		if(this.speedY > this.maxSpeedY)
			this.speedY = this.maxSpeedY;
		// Stop if slow enough
		else if (Math.abs(this.speedY) < .01)
			this.speedY = 0;

		// Which way are we facing?
		this.facingRight = (this.speedX >= 0)

		// Update those positions
		this.x += (this.speedX * engine.gs.worldSpeed);
		this.y += (this.speedY * engine.gs.worldSpeed);

		// Did we fall off the bottom?
		if(this.y > 480)
			engine.destroySprite(this.id, true);
	};//end function logic

	this.sprite.collide = function(engine, hit) {
		if(this.currentAnimation == "die")
			return;

		var l1 = this.x + this.collideOffsetX;
		var t1 = this.y + this.collideOffsetY;
		var r1 = l1     + this.collideWidth;
		var b1 = t1     + this.collideHeight;

		var l2 = hit.x + hit.collideOffsetX;
		var t2 = hit.y + hit.collideOffsetY;
		var r2 = l2    + hit.collideWidth;
		var b2 = t2    + hit.collideHeight;

		// If we were were about to land on the sprite...
		if (b1 - this.speedY <= t2) {
			switch (hit.type) {
				case "player":
					engine.destroySprite(hit.id);
				case "scenery":
					break;
				default:
					this.y = t2 - this.height;
					this.speedY = 0;
					break;
			}
		}
		// If we hit the block from the left
		else if (r1 - this.speedX <= l2) {
			switch (hit.type) {
				case "player":
					engine.destroySprite(hit.id);
					break;
				case "scenery":
					break;
				default:
					this.x = l2 - this.width + this.collideOffsetX;
					this.speedX = 0;
					this.facingRight = false;
					break;
			}
		}

		// If we hit the block from the right
		else if (l1 - this.speedX >= r2) {
			switch (hit.type) {
				case "player":
					engine.destroySprite(hit.id);
					break;
				case "scenery":
					break;
				default:
					this.x = r2 - this.collideOffsetX;
					this.speedX = 0;
					this.facingRight = true;
					break;
			}
		}
	};//end collide

	this.sprite.debugRender = function(resource, context, gs, renderPoint) {
		context.fillStyle = "rgba(255, 255, 255, 0.5)";
		context.fillRect(
			renderPoint.x + this.collideOffsetX,
			renderPoint.y + this.collideOffsetY,
			this.collideWidth,
			this.collideHeight
		);
	};//end function render

	return this.sprite;
}//end object