/**
 * The werewolf is just like the greywolf until it turns into a werewolf (below)
 * @author Torgie
 */
function sprite_gp_wolf(p) {

	p.type = "enemy";
	p.name = "gp_wolf";
	p.src = {
		idle: "images/sprites/world1/gp_wolf_idle.png",
		run: "images/sprites/world1/gp_wolf_run.png"
	};
	p.width = 131;
	p.height = 73;

	this.sprite = new sprite(p);

	this.sprite.collideWidth = 111;
	this.sprite.collideHeight = 50;
	this.sprite.collideOffsetX = 15;
	this.sprite.collideOffsetY = 23;

	this.sprite.currentAnimation = "idle";
	this.sprite.animation = {
		idle: {
			frames: 2,
			currentFrame: 0,
			flipEvery: 10,
			flip: 0
		},
		run: {
			frames: 8,
			currentFrame: 0,
			flipEvery: 20,
			flip: 0
		}
	};

	this.sprite.friction = (1 - 0.1);
	this.sprite.acceleration = .75;

	this.sprite.facingRight = false;
	this.sprite.maxSpeedX = 9;
	this.sprite.maxSpeedY = 10;

	this.sprite.destroy = function(engine, i, j) {
		engine.gs.keyControl = false;
		engine.gs.clearKeys();

		var p = engine.getSprite("p1");
		p.speedX = 10;
		p.speedY = -1;
		p.facingRight = false;
		p.setAnimation("worry");

		// Spawn a new big scary werewolf and destroy this one
		engine.declareSprite("game", new sprite_gp_werewolf({
			x: this.x - 2,
			y: this.y - 59,
			transform: true,
			uniqueID: "were"
		}));
		engine.destroySprite(this.id, true);
	};

	// Declare the logic step
	this.sprite.logic = function(engine) {
		if(this.currentAnimation == "idle")
			this.animation.idle.flip += 1;
		else
			this.animation[this.currentAnimation].flip += (Math.abs(this.speedX) * engine.gs.worldSpeed);

		// Attack the player if the player is within 1000 px
		var chasingPlayer = Math.abs(this.x  - engine.gs.playerX) < 500;
		if (chasingPlayer) {
			this.setAnimation("run");
			if (engine.gs.playerX+8 > this.x) 
				this.speedX += this.acceleration;
			else 
				this.speedX -= this.acceleration;
		}

		this.speedY += engine.gs.gravity;

		// Don't run too fast!
		if (this.speedX < 0 && Math.abs(this.speedX) > this.maxSpeedX) 
			this.speedX = (this.maxSpeedX * -1);
		else if (this.speedX > this.maxSpeedX) 
			this.speedX = this.maxSpeedX;
		else if (Math.abs(this.speedX) < .01) 
			this.speedX = 0;

		// Don't go down too fast
		if(this.speedY > this.maxSpeedY)
			this.speedY = this.maxSpeedY;
		// Stop if slow enough
		else if (Math.abs(this.speedY) < .01)
			this.speedY = 0;

		// Which way are we facing?
		this.facingRight = (this.speedX > 0)

		// Slow down if idling
		if(!chasingPlayer)
			this.speedX *= this.friction;

		// Update those positions
		this.x += (this.speedX * engine.gs.worldSpeed);
		this.y += (this.speedY * engine.gs.worldSpeed);

		// Did we fall off the bottom?
		if(this.y > 480)
			engine.destroySprite(this.id, true);
	};//end function logic

	this.sprite.collide = function(engine, hit) {

		var l1 = this.x + this.collideOffsetX;
		var t1 = this.y + this.collideOffsetY;
		var r1 = l1     + this.collideWidth;
		var b1 = t1     + this.collideHeight;

		var l2 = hit.x + hit.collideOffsetX;
		var t2 = hit.y + hit.collideOffsetY;
		var r2 = l2    + hit.collideWidth;
		var b2 = t2    + hit.collideHeight;

		// If we were were about to land on the sprite...
		if (b1 - this.speedY <= t2) {
			switch (hit.type) {
				case "player":
					engine.destroySprite(hit.id);
					break;
				case "scenery":
					break;
				default:
					this.y = t2 - this.height;
					this.speedY = 0;
					break;
			}
		}
		// If we hit the block from the left
		else if (r1 - this.speedX <= l2) {
			switch (hit.type) {
				case "player":
				case "object":
					engine.destroySprite(hit.id);
					break;
				case "scenery":
					break;
				default:
					this.x = l2 - this.width + this.collideOffsetX -10;
					this.speedX = 0;
					break;
			}
		}

		// If we hit the block from the right
		else if (l1 - this.speedX >= r2) {
			switch (hit.type) {
				case "player":
				case "object":
					engine.destroySprite(hit.id);
					break;
				case "scenery":
					break;
				default:
					this.x = r2 - this.collideOffsetX;
					this.speedX = 0;
					break;
			}
		}
	};//end collide

	return this.sprite;
}//end object

/**
 * The werewolf is just like the greywolf until it turns into a werewolfBig (below)
 * @author Torgie
 */
function sprite_gp_werewolf(p) {

	p.type = "enemy";
	p.name = "gp_werewolf";
	p.src = {
		transform: "images/sprites/world1/gp_werewolf_transform.png",
		run: "images/sprites/world1/gp_werewolf_walk.png"
	};
	p.width = 131;
	p.height = 131;

	this.sprite = new sprite(p);

	this.sprite.collideWidth = 131;
	this.sprite.collideHeight = 131;
	this.sprite.collideOffsetX = 0;
	this.sprite.collideOffsetY = 0;

	this.sprite.currentAnimation = (p.transform === true) ? "transform" : "run";
	this.sprite.animation = {
		transform: {
			frames: 10,
			currentFrame: 0,
			flipEvery: 3,
			flip: 0
		},
		run: {
			frames: 8,
			currentFrame: 0,
			flipEvery: 15,
			flip: 0
		}
	};

	this.sprite.friction = (1 - 0.1);
	this.sprite.acceleration = .25;

	this.sprite.facingRight = false;
	this.sprite.maxSpeedX = 8;
	this.sprite.maxSpeedY = 10;

	this.sprite.transformFrames = 0;

	this.sprite.destroy = function(engine, i, j) {
		engine.destroySprite(this.id, true);
	};

	// Declare the logic step
	this.sprite.logic = function(engine) {
		if(this.currentAnimation == "transform") {
			this.animation.transform.flip += 1;
			if (this.transformFrames++ >= 30) {
				engine.gs.keyControl = true;
				this.setAnimation("run");
			}
			return;
		}
		else
			this.animation[this.currentAnimation].flip += (Math.abs(this.speedX) * engine.gs.worldSpeed);

		// Attack the player if the player is within 1000 px
		var chasingPlayer = Math.abs(this.x  - engine.gs.playerX) < 1000;
		if (chasingPlayer) {
			this.setAnimation("run");
			if (engine.gs.playerX > this.x) 
				this.speedX += this.acceleration;
			else 
				this.speedX -= this.acceleration;
		}

		this.speedY += engine.gs.gravity;

		// Don't run too fast!
		if (this.speedX < 0 && Math.abs(this.speedX) > this.maxSpeedX) 
			this.speedX = (this.maxSpeedX * -1);
		else if (this.speedX > this.maxSpeedX) 
			this.speedX = this.maxSpeedX;
		else if (Math.abs(this.speedX) < .01) 
			this.speedX = 0;

		// Don't go down too fast
		if(this.speedY > this.maxSpeedY)
			this.speedY = this.maxSpeedY;
		// Stop if slow enough
		else if (Math.abs(this.speedY) < .01)
			this.speedY = 0;

		// Which way are we facing?
		this.facingRight = (this.speedX >= 0)

		// Slow down if idling
		if(!chasingPlayer)
			this.speedX *= this.friction;

		// Update those positions
		this.x += (this.speedX * engine.gs.worldSpeed);
		this.y += (this.speedY * engine.gs.worldSpeed);

		// Did we fall off the bottom?
		if(this.y > 480)
			engine.destroySprite(this.id, true);
	};//end function logic

	this.sprite.collide = function(engine, hit) {

		var l1 = this.x + this.collideOffsetX;
		var t1 = this.y + this.collideOffsetY;
		var r1 = l1     + this.collideWidth;
		var b1 = t1     + this.collideHeight;

		var l2 = hit.x + hit.collideOffsetX;
		var t2 = hit.y + hit.collideOffsetY;
		var r2 = l2    + hit.collideWidth;
		var b2 = t2    + hit.collideHeight;

		// If we were were about to land on the sprite...
		if (b1 - this.speedY <= t2) {
			switch (hit.type) {
				case "player":
				case "object":
					engine.destroySprite(hit.id);
					break;
				default:
					this.y = t2 - this.height;
					this.speedY = 0;
					break;
			}
		}
		// If we hit the block from the left
		else if (r1 - this.speedX <= l2) {
			switch (hit.type) {
				case "player":
				case "object":
					engine.destroySprite(hit.id);
					break;
				default:
					this.x = l2 - this.width + this.collideOffsetX;
					this.speedX = 0;
					break;
			}
		}

		// If we hit the block from the right
		else if (l1 - this.speedX >= r2) {
			switch (hit.type) {
				case "player":
				case "object":
					engine.destroySprite(hit.id);
					break;
				default:
					this.x = r2 - this.collideOffsetX;
					this.speedX = 0;
					break;
			}
		}
	};//end collide

	this.sprite.debugRender = function(resource, context, gs, renderPoint) {
		context.fillStyle = "rgba(255, 255, 255, 0.5)";
		context.fillRect(
			renderPoint.x + this.collideOffsetX,
			renderPoint.y + this.collideOffsetY,
			this.collideWidth,
			this.collideHeight
		);
	};//end function render

	return this.sprite;
}//end object